--Sprites from itch.io: "Kings and Pigs"

local sti = require "libraries/sti"
local cam = require "libraries/camera"
local anim8 = require "libraries/anim8"

function love.load()
    map = sti("maps/map.lua")
	camera = cam()
	
	windowWidth = love.graphics.getWidth()
	windowHeight = love.graphics.getHeight()
	mapScale = 2;

	world = love.physics.newWorld(0, 500)
	love.physics.setMeter(64)
	love.graphics.setDefaultFilter("nearest", "nearest")

	player = {}

	player.fallImg = love.graphics.newImage("sprites/player_fall.png")
	player.groundImg = love.graphics.newImage("sprites/player_ground.png")
	player.jumpImg = love.graphics.newImage("sprites/player_jump.png")

	player.spritesheet = love.graphics.newImage("sprites/spritesheet.png")
	player.animGrid = anim8.newGrid(38, 28, player.spritesheet:getWidth(), player.spritesheet:getHeight())

	player.animations = {}
	player.animations.idleRight = anim8.newAnimation(player.animGrid('1-12', 1), 0.1)
	player.animations.idleLeft = anim8.newAnimation(player.animGrid('1-12', 2), 0.1)
	player.animations.runLeft = anim8.newAnimation(player.animGrid('1-6', 3), 0.1)
	player.animations.runRight = anim8.newAnimation(player.animGrid('1-6', 4), 0.1)
	player.anim = player.animations.idleLeft

	player.x = windowWidth/2
	player.y = windowHeight/2
	player.w = 75
	player.h = 80
	player.dir = nil
	player.maxVelX = 400
	player.maxVelY = 800
	player.speed = 700
	player.canJump = true
	player.jumpForce = 1000
	player.body = love.physics.newBody(world, player.x, player.y, "dynamic")
	player.shape = love.physics.newRectangleShape(player.w, player.h)
	player.fixture = love.physics.newFixture(player.body, player.shape)
	player.body:setFixedRotation(true)

	walls = {}
	if (map.layers["Collidable"]) then
		for i, obj in pairs(map.layers["Collidable"].objects) do
			--local wall = love.physics.newBody(world, obj.x, obj.y, "static")
			--table.insert(walls, wall)
			walls.body = love.physics.newBody(world, (obj.x+obj.width/2)*mapScale, (obj.y+obj.height/2)*mapScale, "static")
			walls.shape = love.physics.newRectangleShape(obj.width*mapScale, obj.height*mapScale)
			walls.fixture = love.physics.newFixture(walls.body, walls.shape)
		end
	end	
end

function love.update(dt)
	player.velX, player.velY = player.body:getLinearVelocity()
	-- Get width/height of background
	local mapW = (map.width * map.tilewidth)*mapScale
	local mapH = (map.height * map.tileheight)*mapScale
	local isMoving = false

	if love.keyboard.isDown("right") then
        player.body:applyForce(player.speed, 0)
		player.dir = "right"
		player.anim = player.animations.runRight
		isMoving = true
    end

	if love.keyboard.isDown("left") then
		player.dir = "left"
        player.body:applyForce(-player.speed, 0)
		player.anim = player.animations.runLeft
		isMoving = true
    end

	if isMoving == false then
		if player.dir == "right" then
			player.anim = player.animations.idleRight
		end
		if player.dir == "left" then
			player.anim = player.animations.idleLeft
		end
	end

	camera:lookAt(player.body:getX(), player.body:getY())

    if camera.x < windowWidth/2 then
        camera.x = windowWidth/2
    end
    if camera.y < windowHeight/2 then
        camera.y = windowHeight/2
    end

    if camera.x > (mapW - windowWidth/2) then
        camera.x = (mapW - windowWidth/2)
    end
    if camera.y > (mapH - windowHeight/2) then
        camera.y = (mapH - windowHeight/2)
    end

	player.anim:update(dt)

	if (player.velY == 0) then
		player.canJump = true
	end

	player.body:setLinearVelocity(math.min(player.velX, player.maxVelX), math.min(player.velY, player.maxVelY))
	world:update(dt)
end

function love.draw()
	
	camera:attach()
		love.graphics.push()
		love.graphics.scale(mapScale)
		map:drawLayer(map.layers["Walls"])
		map:drawLayer(map.layers["Blocks"])
		love.graphics.pop()

		--Draw hitbox (optional)
		--love.graphics.polygon("fill", player.body:getWorldPoints(player.shape:getPoints()))

		--Allign hitbox/player body to animation
		player.anim:draw(player.spritesheet, player.body:getX()-player.w-5, player.body:getY()-player.h+10, nil, 4)
	camera:detach()

	love.graphics.print("FPS: "..tostring(love.timer.getFPS()), 10, 10)
end

function love.keypressed(key)
	if key == "up" and player.canJump then
		player.body:applyLinearImpulse(0, -player.jumpForce)
		player.canJump = false
	end
end